@@ -32,10 +32,13 @@ impl cryptraits::hmac::Hmac for Hmac {
     }
 }
 
+impl cryptraits::convert::Len for Hmac {
+    const LEN: usize = 32;
+}
+
 #[cfg(test)]
 mod tests {
     use cryptraits::hmac::Hmac as _;
-    use hex_literal::hex;
 
     use super::Hmac;
 
@@ -46,12 +49,9 @@ mod tests {
 
         mac.update(b"input message");
 
-        let code_bytes = hex!(
-            "
-                97d2a569059bbcd8ead4444ff99071f4
-                c01d005bcefe0d3567e1be628e5fdcd9
-            "
-        );
+        let code_bytes =
+            hex::decode("97d2a569059bbcd8ead4444ff99071f4c01d005bcefe0d3567e1be628e5fdcd9")
+                .unwrap();
 
         assert!(mac.verify_slice(&code_bytes[..]).is_ok());
     }
